% -------------------------------------------------------------------------
% Copyright  2017, Texas State University. All rights reserved.
% -------------------------------------------------------------------------
%
% This work is licensed under a Creative Commons Attribution-NonCommercial
% -ShareAlike 3.0 United States License. See license conditions at:
% https://creativecommons.org/licenses/by-nc-sa/3.0/us/
%
% For all other uses, please contact the Office for Commercialization and 
% Industry Relations at Texas State University http://www.txstate.edu/ocir/
%
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% Created by: Ioannis Rigas (rigas.ioann@gmail.com)
% -------------------------------------------------------------------------


function [headers, features] = extract_fixation_features(ET, Params)

% Initialize empty headers and features vectors 
features = [];
headers = cell(1, 1);

% Initialize handles for descriptive statistics functions and headers
stat_fncs = @(x)[nanmean(x), nanmedian(x), nanstd(x), iqr(x), skewness(x), kurtosis(x)];
stat_headers = @(x){strcat(x, '_Mn'), strcat(x, '_Md'), strcat(x, '_Sd'), ...
                   strcat(x, '_Iq'), strcat(x, '_Sk'), strcat(x, '_Ku')};

% Get number of fixations
Fix_Num = ET.Scalars.Data.N_Fixations;
     
% Calculate overall fixation velocity statistics prior signal segmentation
% (needed for features: Fix_PrAbP90VelThr_R, Fix_PrCrP90VelThr_R)
v = sqrt(ET.Vectors.velX.^2 + ET.Vectors.velY.^2);
possibleFixationIdx = v < Params.ExcludeAboveThisVelForNoiseAnalysis;
FixBlocks = bwlabel(possibleFixationIdx);
N_Potential_Fixations = max(FixBlocks);
central_samples = [Params.minFixDur]*[Params.samplingFreq]/6;
each_fix_samples = cell(N_Potential_Fixations, 1);

for k = 1:N_Potential_Fixations

    fixIdx = find(FixBlocks == k);
    if(length(fixIdx)/[Params.samplingFreq] < Params.minFixDur), continue, end;
    each_fix_samples{k} = v(floor(fixIdx(1)+central_samples):ceil(fixIdx(end)-central_samples));

end

all_fix_samples = cell2mat(each_fix_samples);
Fix_Vel_P90 = prctile(all_fix_samples(:), 90);

% Extract fixation position, velocity, and acceleration profiles
Fix_Start = zeros(Fix_Num, 1);
Fix_End = zeros(Fix_Num, 1);
Fix_Pos_H = cell(Fix_Num, 1);
Fix_Pos_V = cell(Fix_Num, 1);
Fix_Vel_H = cell(Fix_Num, 1);
Fix_Vel_V = cell(Fix_Num, 1);
Fix_Acc_H = cell(Fix_Num, 1);
Fix_Acc_V = cell(Fix_Num, 1);

for i = 1:Fix_Num
    
    % Convert from time to samples
    Fix_Start(i) = round(ET.Scalars.Data.FixationInfo(i).start*Params.samplingFreq);
    Fix_End(i) = round(ET.Scalars.Data.FixationInfo(i).end*Params.samplingFreq);
    
    % Extract profiles
    Fix_Pos_H{i} = ET.Vectors.Xsmo(Fix_Start(i):Fix_End(i));
    Fix_Pos_V{i} = ET.Vectors.Ysmo(Fix_Start(i):Fix_End(i));
    Fix_Vel_H{i} = ET.Vectors.velX(Fix_Start(i):Fix_End(i));
    Fix_Vel_V{i} = ET.Vectors.velY(Fix_Start(i):Fix_End(i));
    Fix_Acc_H{i} = ET.Vectors.accX(Fix_Start(i):Fix_End(i));
    Fix_Acc_V{i} = ET.Vectors.accY(Fix_Start(i):Fix_End(i)); 
    
end

% Calculate feature values from multiple fixation instances
Fix_Dur = zeros(Fix_Num, 1);
Fix_PosCentroid_H = zeros(Fix_Num, 1);
Fix_PosCentroid_V = zeros(Fix_Num, 1);
Fix_DriftDisp_H = zeros(Fix_Num, 1);
Fix_DriftDisp_V = zeros(Fix_Num, 1);
Fix_DriftDisp_R = zeros(Fix_Num, 1);
Fix_DriftDist_H = zeros(Fix_Num, 1);
Fix_DriftDist_V = zeros(Fix_Num, 1);
Fix_DriftDist_R = zeros(Fix_Num, 1);
Fix_DriftAvgSpeed_H = zeros(Fix_Num, 1);
Fix_DriftAvgSpeed_V = zeros(Fix_Num, 1);
Fix_DriftAvgSpeed_R = zeros(Fix_Num, 1);
Fix_DriftFitLn_Slope_H = zeros(Fix_Num, 1);
Fix_DriftFitLn_Slope_V = zeros(Fix_Num, 1);
Fix_DriftFitLn_R2_H = zeros(Fix_Num, 1);
Fix_DriftFitLn_R2_V = zeros(Fix_Num, 1);
Fix_DriftFitQd_R2_H = zeros(Fix_Num, 1);
Fix_DriftFitQd_R2_V = zeros(Fix_Num, 1);
Fix_StepLQParam_H = zeros(Fix_Num, 2);
Fix_StepLQParam_V = zeros(Fix_Num, 2);
Fix_VelProfMn_H = zeros(Fix_Num, 1);
Fix_VelProfMn_V = zeros(Fix_Num, 1);
Fix_VelProfMn_R = zeros(Fix_Num, 1);
Fix_VelProfMd_H = zeros(Fix_Num, 1);
Fix_VelProfMd_V = zeros(Fix_Num, 1);
Fix_VelProfMd_R = zeros(Fix_Num, 1);
Fix_VelProfSd_H = zeros(Fix_Num, 1);
Fix_VelProfSd_V = zeros(Fix_Num, 1);
Fix_VelProfSd_R = zeros(Fix_Num, 1);
Fix_VelProfSk_H = zeros(Fix_Num, 1);
Fix_VelProfSk_V = zeros(Fix_Num, 1);
Fix_VelProfSk_R = zeros(Fix_Num, 1);
Fix_VelProfKu_H = zeros(Fix_Num, 1);
Fix_VelProfKu_V = zeros(Fix_Num, 1);
Fix_VelProfKu_R = zeros(Fix_Num, 1);
Fix_PrAbP90VelThr_R = zeros(Fix_Num, 1);
Fix_PrCrP90VelThr_R = zeros(Fix_Num, 1);
Fix_AccProfMn_H = zeros(Fix_Num, 1);
Fix_AccProfMn_V = zeros(Fix_Num, 1);
Fix_AccProfMn_R = zeros(Fix_Num, 1);
Fix_AccProfMd_H = zeros(Fix_Num, 1);
Fix_AccProfMd_V = zeros(Fix_Num, 1);
Fix_AccProfMd_R = zeros(Fix_Num, 1);
Fix_AccProfSd_H = zeros(Fix_Num, 1);
Fix_AccProfSd_V = zeros(Fix_Num, 1);
Fix_AccProfSd_R = zeros(Fix_Num, 1);
Fix_AccProfSk_H = zeros(Fix_Num, 1);
Fix_AccProfSk_V = zeros(Fix_Num, 1);
Fix_AccProfSk_R = zeros(Fix_Num, 1);
Fix_AccProfKu_H = zeros(Fix_Num, 1);
Fix_AccProfKu_V = zeros(Fix_Num, 1);
Fix_AccProfKu_R = zeros(Fix_Num, 1);

for i = 1:Fix_Num
    
    % Duration (ms)
    Fix_Dur(i) = 1000*(Fix_End(i) - Fix_Start(i))/[Params.samplingFreq];
        
	% Position Centroid H, V (deg)
    Fix_PosCentroid_H(i) = mean(Fix_Pos_H{i});
    Fix_PosCentroid_V(i) = mean(Fix_Pos_V{i});	
		
    % Drift Displacement H, V, R (deg)
    Fix_DriftDisp_H(i) = abs(Fix_Pos_H{i}(end) - Fix_Pos_H{i}(1));
    Fix_DriftDisp_V(i) = abs(Fix_Pos_V{i}(end) - Fix_Pos_V{i}(1));
    Fix_DriftDisp_R(i) = sqrt((Fix_Pos_H{i}(end)-Fix_Pos_H{i}(1)).^2 + (Fix_Pos_V{i}(end)-Fix_Pos_V{i}(1)).^2);
        
    % Drift Distance H, V, R (deg)
    Fix_DriftDist_H(i) = sum(abs(diff(Fix_Pos_H{i})));
    Fix_DriftDist_V(i) = sum(abs(diff(Fix_Pos_V{i})));
    Fix_DriftDist_R(i) = sum(sqrt(diff(Fix_Pos_H{i}).^2 + diff(Fix_Pos_V{i}).^2));
        
    % Drift mean Velocity H, V, R (deg/s)
    Fix_DriftAvgSpeed_H(i) = Fix_DriftDisp_H(i)/(0.001*Fix_Dur(i));
    Fix_DriftAvgSpeed_V(i) = Fix_DriftDisp_V(i)/(0.001*Fix_Dur(i));
    Fix_DriftAvgSpeed_R(i) = Fix_DriftDisp_R(i)/(0.001*Fix_Dur(i)); 

    % Drift 1-order (linear) fit Slope and R^2 H, V 
    timeData = (Fix_Start(i):Fix_End(i))'/[Params.samplingFreq];
    Tregress = [ones(length(timeData),1) timeData];
    [b, ~, ~, ~, stats] = regress(Fix_Pos_H{i}, Tregress);
    Fix_DriftFitLn_Slope_H(i) = b(2);
    Fix_DriftFitLn_R2_H(i) = stats(1);
    [b, ~, ~, ~, stats] = regress(Fix_Pos_V{i}, Tregress);
    Fix_DriftFitLn_Slope_V(i) = b(2);
    Fix_DriftFitLn_R2_V(i) = stats(1);

    % Drift 2-order (quadratic) fit R^2 H, V 
    Tregress =[ones(length(timeData),1) timeData power(timeData,2)];
    [~, ~, ~, ~, stats] = regress(Fix_Pos_H{i}, Tregress);
    Fix_DriftFitQd_R2_H(i) = stats(1);
    [~, ~, ~, ~, stats] = regress(Fix_Pos_V{i}, Tregress);
    Fix_DriftFitQd_R2_V(i) = stats(1);

    % Drift step-wise fit parameter percentages H, V
    Tstep = [timeData power(timeData,2)];
    [~, ~, ~, Xinmodel, ~, ~, ~] = stepwisefit(Tstep, Fix_Pos_H{i}, 'display', 'off');
    [~, ~, ~, Yinmodel, ~, ~, ~] = stepwisefit(Tstep, Fix_Pos_V{i}, 'display', 'off');
    Fix_StepLQParam_H(i, 1) = Xinmodel(1); Fix_StepLQParam_H(i, 2) = Xinmodel(2);
    Fix_StepLQParam_V(i, 1) = Yinmodel(1); Fix_StepLQParam_V(i, 2) = Yinmodel(2);
   
    % Velocity Profile Mean H, V, R (deg/s)
    Fix_VelProfMn_H(i) = mean(abs(Fix_Vel_H{i}));
    Fix_VelProfMn_V(i) = mean(abs(Fix_Vel_V{i}));
    Fix_VelProfMn_R(i) = mean(sqrt(Fix_Vel_H{i}.^2 + Fix_Vel_V{i}.^2));
        
    % Velocity Profile Median H, V, R (deg/s)
    Fix_VelProfMd_H(i) = median(abs(Fix_Vel_H{i}));
    Fix_VelProfMd_V(i) = median(abs(Fix_Vel_V{i}));
    Fix_VelProfMd_R(i) = median(sqrt(Fix_Vel_H{i}.^2 + Fix_Vel_V{i}.^2));
        
    % Velocity Profile Std H, V, R (deg/s)
    Fix_VelProfSd_H(i) = std(abs(Fix_Vel_H{i}));
    Fix_VelProfSd_V(i) = std(abs(Fix_Vel_V{i}));
    Fix_VelProfSd_R(i) = std(sqrt(Fix_Vel_H{i}.^2 + Fix_Vel_V{i}.^2));
        
    % Velocity Profile Skewness H, V, R (deg/s)
    Fix_VelProfSk_H(i) = skewness(abs(Fix_Vel_H{i}));
    Fix_VelProfSk_V(i) = skewness(abs(Fix_Vel_V{i}));
    Fix_VelProfSk_R(i) = skewness(sqrt(Fix_Vel_H{i}.^2 + Fix_Vel_V{i}.^2));
        
    % Velocity Profile Kurtosis H, V, R (deg/s)
    Fix_VelProfKu_H(i) = kurtosis(abs(Fix_Vel_H{i}));
    Fix_VelProfKu_V(i) = kurtosis(abs(Fix_Vel_V{i}));
    Fix_VelProfKu_R(i) = kurtosis(sqrt(Fix_Vel_H{i}.^2 + Fix_Vel_V{i}.^2));
        
	% Percent Above 90-percentile Velocity Threshold R
    v = sqrt(ET.Vectors.velX.^2 + ET.Vectors.velY.^2);
    Fix_PrAbP90VelThr_R(i) = 100*sum(v(Fix_Start(i):Fix_End(i)) > Fix_Vel_P90)/length(v(Fix_Start(i):Fix_End(i)));
       
    % Percent Crossing 90-percentile Velocity Threshold R
    cross_idx = crossing(v(Fix_Start(i):Fix_End(i)), [], Fix_Vel_P90);
    Fix_PrCrP90VelThr_R(i) = 100*length(cross_idx)/length(v(Fix_Start(i):Fix_End(i)));
		
    % Acceleration Profile Mean H, V, R (deg/s^2)
    Fix_AccProfMn_H(i) = mean(abs(Fix_Acc_H{i}));
    Fix_AccProfMn_V(i) = mean(abs(Fix_Acc_V{i}));
    Fix_AccProfMn_R(i) = mean(sqrt(Fix_Acc_H{i}.^2 + Fix_Acc_V{i}.^2));
        
    % Acceleration Profile Median H, V, R (deg/s^2)
    Fix_AccProfMd_H(i) = median(abs(Fix_Acc_H{i}));
    Fix_AccProfMd_V(i) = median(abs(Fix_Acc_V{i}));
    Fix_AccProfMd_R(i) = median(sqrt(Fix_Acc_H{i}.^2 + Fix_Acc_V{i}.^2));
        
    % Acceleration Profile Std H, V, R (deg/s^2)
    Fix_AccProfSd_H(i) = std(abs(Fix_Acc_H{i}));
    Fix_AccProfSd_V(i) = std(abs(Fix_Acc_V{i}));
    Fix_AccProfSd_R(i) = std(sqrt(Fix_Acc_H{i}.^2 + Fix_Acc_V{i}.^2));
        
    % Acceleration Profile Skewness H, V, R (deg/s^2)
    Fix_AccProfSk_H(i) = skewness(abs(Fix_Acc_H{i}));
    Fix_AccProfSk_V(i) = skewness(abs(Fix_Acc_V{i}));
    Fix_AccProfSk_R(i) = skewness(sqrt(Fix_Acc_H{i}.^2 + Fix_Acc_V{i}.^2));
        
    % Acceleration Profile Kurtosis H, V, R (deg/s^2)
    Fix_AccProfKu_H(i) = kurtosis(abs(Fix_Acc_H{i}));
    Fix_AccProfKu_V(i) = kurtosis(abs(Fix_Acc_V{i}));
    Fix_AccProfKu_R(i) = kurtosis(sqrt(Fix_Acc_H{i}.^2 + Fix_Acc_V{i}.^2));
    
end

% Create indices to filter out fixations
fIdx = true(Fix_Num, 1);
for i = 1:length(fIdx)

    if(Fix_Dur(i) < Params.minFixDur)
        fIdx = false;
    end

end

% Calculate final fixation features (modeling, descriptive statistics)

% Fixation Rate (number of fixations per second)
currFeature = length(find(fIdx))/([ET.Scalars.Data.Length]/[Params.samplingFreq]);
features = [features, currFeature];
headers{1, 1} = 'Fix_Rate';

% Fixation Duration
currFeature = Fix_Dur(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_Dur')];

% Fixation Position Centroid H, V
currFeature = Fix_PosCentroid_H(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_PosCentroid_H')];
currFeature = Fix_PosCentroid_V(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_PosCentroid_V')];

% Fixation Drift Displacement H, V, R
currFeature = Fix_DriftDisp_H(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_DriftDisp_H')];
currFeature = Fix_DriftDisp_V(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_DriftDisp_V')];
currFeature = Fix_DriftDisp_R(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_DriftDisp_R')];

% Fixation Drift Distance H, V, R
currFeature = Fix_DriftDist_H(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_DriftDist_H')];
currFeature = Fix_DriftDist_V(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_DriftDist_V')];
currFeature = Fix_DriftDist_R(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_DriftDist_R')];

% Fixation Drift Average Speed H, V, R
currFeature = Fix_DriftAvgSpeed_H(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_DriftAvgSpeed_H')];
currFeature = Fix_DriftAvgSpeed_V(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_DriftAvgSpeed_V')];
currFeature = Fix_DriftAvgSpeed_R(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_DriftAvgSpeed_R')];

% Fixation Drift 1-order (linear) Fit Slope H, V
currFeature = Fix_DriftFitLn_Slope_H(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_DriftFitLn_Slope_H')];
currFeature = Fix_DriftFitLn_Slope_V(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_DriftFitLn_Slope_V')];

% Fixation Drift 1-order (linear) Fit R^2 H, V
currFeature = Fix_DriftFitLn_R2_H(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_DriftFitLn_R2_H')];
currFeature = Fix_DriftFitLn_R2_V(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_DriftFitLn_R2_V')];

% Fixation Drift 2-order (quadratic) Fit R^2 H, V
currFeature = Fix_DriftFitQd_R2_H(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_DriftFitQd_R2_H')];
currFeature = Fix_DriftFitQd_R2_V(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_DriftFitQd_R2_V')];

% Fixation Drift Step-wise Fit L0Q0 Parameter Percents H, V
currFeature = 100 * mean(~Fix_StepLQParam_H(fIdx, 1) & ~Fix_StepLQParam_H(fIdx, 2));
features = [features, currFeature];
headers = [headers, 'Fix_DriftPrL0Q0_H'];
currFeature = 100 * mean(~Fix_StepLQParam_V(fIdx, 1) & ~Fix_StepLQParam_V(fIdx, 2));
features = [features, currFeature];
headers = [headers, 'Fix_DriftPrL0Q0_V'];

% Fixation Drift Step-wise Fit L1Q0 Parameter Percents H, V
currFeature = 100 * mean(Fix_StepLQParam_H(fIdx, 1) & ~Fix_StepLQParam_H(fIdx, 2));
features = [features, currFeature];
headers = [headers, 'Fix_DriftPrL1Q0_H'];
currFeature = 100 * mean(Fix_StepLQParam_V(fIdx, 1) & ~Fix_StepLQParam_V(fIdx, 2));
features = [features, currFeature];
headers = [headers, 'Fix_DriftPrL1Q0_V'];

% Fixation Drift Step-wise Fit L0Q1 Parameter Percents H, V
currFeature = 100 * mean(~Fix_StepLQParam_H(fIdx, 1) & Fix_StepLQParam_H(fIdx, 2));
features = [features, currFeature];
headers = [headers, 'Fix_DriftPrL0Q1_H'];
currFeature = 100 * mean(~Fix_StepLQParam_V(fIdx, 1) & Fix_StepLQParam_V(fIdx, 2));
features = [features, currFeature];
headers = [headers, 'Fix_DriftPrL0Q1_V'];

% Fixation Drift Step-wise Fit L1Q1 Parameter Percents H, V
currFeature = 100 * mean(Fix_StepLQParam_H(fIdx, 1) & Fix_StepLQParam_H(fIdx, 2));
features = [features, currFeature];
headers = [headers, 'Fix_DriftPrL1Q1_H'];
currFeature = 100 * mean(Fix_StepLQParam_V(fIdx, 1) & Fix_StepLQParam_V(fIdx, 2));
features = [features, currFeature];
headers = [headers, 'Fix_DriftPrL1Q1_V'];

% Fixation Velocity Profile Mean H, V, R
currFeature = Fix_VelProfMn_H(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_VelProfMn_H')];
currFeature = Fix_VelProfMn_V(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_VelProfMn_V')];
currFeature = Fix_VelProfMn_R(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_VelProfMn_R')];

% Fixation Velocity Profile Median H, V, R
currFeature = Fix_VelProfMd_H(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_VelProfMd_H')];
currFeature = Fix_VelProfMd_V(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_VelProfMd_V')];
currFeature = Fix_VelProfMd_R(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_VelProfMd_R')];

% Fixation Velocity Profile Std H, V, R
currFeature = Fix_VelProfSd_H(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_VelProfSd_H')];
currFeature = Fix_VelProfSd_V(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_VelProfSd_V')];
currFeature = Fix_VelProfSd_R(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_VelProfSd_R')];

% Fixation Velocity Profile Skewness H, V, R
currFeature = Fix_VelProfSk_H(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_VelProfSk_H')];
currFeature = Fix_VelProfSk_V(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_VelProfSk_V')];
currFeature = Fix_VelProfSk_R(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_VelProfSk_R')];

% Fixation Velocity Profile Kurtosis H, V, R
currFeature = Fix_VelProfKu_H(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_VelProfKu_H')];
currFeature = Fix_VelProfKu_V(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_VelProfKu_V')];
currFeature = Fix_VelProfKu_R(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_VelProfKu_R')];

% Fixation Percent Above 90-percentile Velocity Threshold R
currFeature = Fix_PrAbP90VelThr_R(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_PrAbP90VelThr_R')];

% Fixation Percent Cross 90-percentile Velocity Threshold R
currFeature = Fix_PrCrP90VelThr_R(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_PrCrP90VelThr_R')];

% Fixation Acceleration Profile Mean H, V, R
currFeature = Fix_AccProfMn_H(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_AccProfMn_H')];
currFeature = Fix_AccProfMn_V(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_AccProfMn_V')];
currFeature = Fix_AccProfMn_R(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_AccProfMn_R')];

% Fixation Acceleration Profile Median H, V, R
currFeature = Fix_AccProfMd_H(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_AccProfMd_H')];
currFeature = Fix_AccProfMd_V(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_AccProfMd_V')];
currFeature = Fix_AccProfMd_R(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_AccProfMd_R')];

% Fixation Acceleration Profile Std H, V, R
currFeature = Fix_AccProfSd_H(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_AccProfSd_H')];
currFeature = Fix_AccProfSd_V(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_AccProfSd_V')];
currFeature = Fix_AccProfSd_R(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_AccProfSd_R')];

% Fixation Acceleration Profile Skewness H, V, R
currFeature = Fix_AccProfSk_H(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_AccProfSk_H')];
currFeature = Fix_AccProfSk_V(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_AccProfSk_V')];
currFeature = Fix_AccProfSk_R(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_AccProfSk_R')];

% Fixation Acceleration Profile Kurtosis H, V, R
currFeature = Fix_AccProfKu_H(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_AccProfKu_H')];
currFeature = Fix_AccProfKu_V(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_AccProfKu_V')];
currFeature = Fix_AccProfKu_R(fIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Fix_AccProfKu_R')];

end