% -------------------------------------------------------------------------
% Copyright  2017, Texas State University. All rights reserved.
% -------------------------------------------------------------------------
%
% This work is licensed under a Creative Commons Attribution-NonCommercial
% -ShareAlike 3.0 United States License. See license conditions at:
% https://creativecommons.org/licenses/by-nc-sa/3.0/us/
%
% For all other uses, please contact the Office for Commercialization and 
% Industry Relations at Texas State University http://www.txstate.edu/ocir/
%
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% Created by: Ioannis Rigas (rigas.ioann@gmail.com)
% -------------------------------------------------------------------------


function [headers, features] = extract_saccade_pso_features(ET, Params)

% ----------------
% Saccade Features
% ----------------

% Initialize empty headers and features vectors 
features = [];
headers = cell(1, 1);

% Initialize handles for descriptive statistics functions and headers
stat_fncs = @(x)[nanmean(x), nanmedian(x), nanstd(x), iqr(x), skewness(x), kurtosis(x)];
stat_headers = @(x){strcat(x, '_Mn'), strcat(x, '_Md'), strcat(x, '_Sd'), ...
                   strcat(x, '_Iq'), strcat(x, '_Sk'), strcat(x, '_Ku')};

% Get number of saccades
Sac_Num = ET.Scalars.Data.N_Saccades;
              
% Extract saccade position, velocity, and acceleration profiles
Sac_Start = zeros(Sac_Num, 1);
Sac_End = zeros(Sac_Num, 1);
Sac_Pos_H_Raw = cell(Sac_Num, 1);
Sac_Pos_V_Raw = cell(Sac_Num, 1);
Sac_Pos_H = cell(Sac_Num, 1);
Sac_Pos_V = cell(Sac_Num, 1);
Sac_Vel_H = cell(Sac_Num, 1);
Sac_Vel_V = cell(Sac_Num, 1);
Sac_Acc_H = cell(Sac_Num, 1);
Sac_Acc_V = cell(Sac_Num, 1);

for i = 1:Sac_Num
        
        % Convert from time to samples
        Sac_Start(i) = round(ET.Scalars.Data.SaccadeInfo(i).start * Params.samplingFreq);
        Sac_End(i) = round(ET.Scalars.Data.SaccadeInfo(i).end * Params.samplingFreq);
        % Extract profiles
        Sac_Pos_H_Raw{i} = ET.Vectors.Xorg(Sac_Start(i):Sac_End(i));
        Sac_Pos_V_Raw{i} = ET.Vectors.Yorg(Sac_Start(i):Sac_End(i));
        Sac_Pos_H{i} = ET.Vectors.Xsmo(Sac_Start(i):Sac_End(i));
        Sac_Pos_V{i} = ET.Vectors.Ysmo(Sac_Start(i):Sac_End(i));
        Sac_Vel_H{i} = ET.Vectors.velX(Sac_Start(i):Sac_End(i));
        Sac_Vel_V{i} = ET.Vectors.velY(Sac_Start(i):Sac_End(i));
        Sac_Acc_H{i} = ET.Vectors.accX(Sac_Start(i):Sac_End(i));
        Sac_Acc_V{i} = ET.Vectors.accY(Sac_Start(i):Sac_End(i));
        
end

% Calculate feature values from multiple saccade instances
Sac_Dur = zeros(Sac_Num, 1);
Sac_Amp_H = zeros(Sac_Num, 1);
Sac_Amp_V = zeros(Sac_Num, 1);
Sac_Amp_R = zeros(Sac_Num, 1);
Sac_Direction_H = zeros(Sac_Num, 1);
Sac_TravDist = zeros(Sac_Num, 1);
Sac_Efficiency = zeros(Sac_Num, 1);
Sac_TailEfficiency = zeros(Sac_Num, 1);
Sac_TailPrInConsist = zeros(Sac_Num, 1);
Sac_InitDir = zeros(Sac_Num, 1);
Sac_InitAvgDev = zeros(Sac_Num, 1);
Sac_MaxRawDev = zeros(Sac_Num, 1);
Sac_PoiMaxRawDev = zeros(Sac_Num, 1);
Sac_AreaCurv = zeros(Sac_Num, 1);
Sac_QuadCurv = zeros(Sac_Num, 1);
Sac_CubCurvM1 = zeros(Sac_Num, 1);
Sac_PoiCubCurvM1 = zeros(Sac_Num, 1);
Sac_CubCurvM2 = zeros(Sac_Num, 1);
Sac_PoiCubCurvM2 = zeros(Sac_Num, 1);
Sac_CubCurvMax = zeros(Sac_Num, 1);
Sac_PoiCubCurvMax = zeros(Sac_Num, 1);
Sac_NumVelLocMin = zeros(Sac_Num, 1);
Sac_PkVel_H = zeros(Sac_Num, 1);
Sac_PkVel_V = zeros(Sac_Num, 1);
Sac_PkVel_R = zeros(Sac_Num, 1);
Sac_MnVel_H = zeros(Sac_Num, 1);
Sac_MnVel_V = zeros(Sac_Num, 1);
Sac_MnVel_R = zeros(Sac_Num, 1);
Sac_VelProfMn_H = zeros(Sac_Num, 1);
Sac_VelProfMn_V = zeros(Sac_Num, 1);
Sac_VelProfMn_R = zeros(Sac_Num, 1);
Sac_VelProfMd_H = zeros(Sac_Num, 1);
Sac_VelProfMd_V = zeros(Sac_Num, 1);
Sac_VelProfMd_R = zeros(Sac_Num, 1);
Sac_VelProfSd_H = zeros(Sac_Num, 1);
Sac_VelProfSd_V = zeros(Sac_Num, 1);
Sac_VelProfSd_R = zeros(Sac_Num, 1);
Sac_VelProfSk_H = zeros(Sac_Num, 1);
Sac_VelProfSk_V = zeros(Sac_Num, 1);
Sac_VelProfSk_R = zeros(Sac_Num, 1);
Sac_VelProfKu_H = zeros(Sac_Num, 1);
Sac_VelProfKu_V = zeros(Sac_Num, 1);
Sac_VelProfKu_R = zeros(Sac_Num, 1);
Sac_PkAcc_H = zeros(Sac_Num, 1);
Sac_PkAcc_V = zeros(Sac_Num, 1);
Sac_PkAcc_R = zeros(Sac_Num, 1);
Sac_PkDec_H = zeros(Sac_Num, 1);
Sac_PkDec_V = zeros(Sac_Num, 1);
Sac_PkDec_R = zeros(Sac_Num, 1);
Sac_AccProfMn_H = zeros(Sac_Num, 1);
Sac_AccProfMn_V = zeros(Sac_Num, 1);
Sac_AccProfMn_R = zeros(Sac_Num, 1);
Sac_AccProfMd_H = zeros(Sac_Num, 1);
Sac_AccProfMd_V = zeros(Sac_Num, 1);
Sac_AccProfMd_R = zeros(Sac_Num, 1);
Sac_AccProfSd_H = zeros(Sac_Num, 1);
Sac_AccProfSd_V = zeros(Sac_Num, 1);
Sac_AccProfSd_R = zeros(Sac_Num, 1);
Sac_AccProfSk_H = zeros(Sac_Num, 1);
Sac_AccProfSk_V = zeros(Sac_Num, 1);
Sac_AccProfSk_R = zeros(Sac_Num, 1);
Sac_AccProfKu_H = zeros(Sac_Num, 1);
Sac_AccProfKu_V = zeros(Sac_Num, 1);
Sac_AccProfKu_R = zeros(Sac_Num, 1);
Sac_AmpDur_Ratio_H = zeros(Sac_Num, 1);
Sac_AmpDur_Ratio_V = zeros(Sac_Num, 1);
Sac_AmpDur_Ratio_R = zeros(Sac_Num, 1);
Sac_PkVelAmp_Ratio_H = zeros(Sac_Num, 1);
Sac_PkVelAmp_Ratio_V = zeros(Sac_Num, 1);
Sac_PkVelAmp_Ratio_R = zeros(Sac_Num, 1);
Sac_PkVelDur_Ratio_H = zeros(Sac_Num, 1);
Sac_PkVelDur_Ratio_V = zeros(Sac_Num, 1);
Sac_PkVelDur_Ratio_R = zeros(Sac_Num, 1);
Sac_PkVelMnVel_Ratio_H = zeros(Sac_Num, 1);
Sac_PkVelMnVel_Ratio_V = zeros(Sac_Num, 1);
Sac_PkVelMnVel_Ratio_R = zeros(Sac_Num, 1);
Sac_PkVelLocNoise_Ratio_R = zeros(Sac_Num, 1);
Sac_AccDecDur_Ratio = zeros(Sac_Num, 1);
Sac_PkAccPkDec_Ratio_H = zeros(Sac_Num, 1);
Sac_PkAccPkDec_Ratio_V = zeros(Sac_Num, 1);
Sac_PkAccPkDec_Ratio_R = zeros(Sac_Num, 1);

for i = 1:Sac_Num
            
    % Duration (ms)
    Sac_Dur(i) = 1000*(Sac_End(i) - Sac_Start(i))/[Params.samplingFreq];
        
    % Amplitude H, V, R (deg)
    Sac_Amp_H(i) = abs(Sac_Pos_H{i}(end) - Sac_Pos_H{i}(1));
    Sac_Amp_V(i) = abs(Sac_Pos_V{i}(end) - Sac_Pos_V{i}(1)); 
    Sac_Amp_R(i) = sqrt(Sac_Amp_H(i).^2 + Sac_Amp_V(i).^2); 
        
    % Saccade Horizontal Direction (used later for reading behavior features)
    Sac_Direction_H(i) = sign(Sac_Pos_H{i}(end) - Sac_Pos_H{i}(1)); % (+1 -> right, -1 -> left)
    
    % Saccade Travelled Distance
    Sac_TravDist(i) = sum(sqrt(diff(Sac_Pos_H{i}).^2 + diff(Sac_Pos_V{i}).^2));
        
    % Saccade Efficiency
    Sac_Efficiency(i) = Sac_Amp_R(i)/Sac_TravDist(i);
        
    % Saccade Tail Efficiency (Tail = last 7 ms)
    Tail_H = Sac_Pos_H_Raw{i}(end-round(0.007*Params.samplingFreq):end); % Use raw signal
    Tail_V = Sac_Pos_V_Raw{i}(end-round(0.007*Params.samplingFreq):end); % Use raw signal
    Tail_Amp = sqrt((Tail_H(end) - Tail_H(1)).^2 + (Tail_V(end) - Tail_V(1)).^2);
    Tail_TravDist = sum(sqrt(diff(Tail_H).^2 + diff(Tail_V).^2));
    Sac_TailEfficiency(i) = Tail_Amp/Tail_TravDist; 
        
    % Saccade Tail Percent Inconsistent (Tail = last 7 ms)
    ConsistentArray = zeros(length(Tail_H)-1, 1);
    for m = 1:length(Tail_H)-1
        v1 = [Tail_H(m+1), Tail_V(m+1)] - [Tail_H(m), Tail_V(m)];
        v2 = [Sac_Pos_H_Raw{i}(end), Sac_Pos_V_Raw{i}(end)] - [Sac_Pos_H_Raw{i}(1), Sac_Pos_V_Raw{i}(1)]; % Use raw signal
        Angle = atan2d(abs(det([v1', v2'])), dot(v1', v2'));
        if(abs(Angle) < 60)
            ConsistentArray(m) = 1;
        end
    end
    Sac_TailPrInConsist(i) = 100 * (1 - sum(ConsistentArray)/(length(Tail_H)-1));
        
    % Saccade Trajectory Curvature Features (using C. Ludwig's function "CurveMetrics.m")
    samples = [ones(length(Sac_Pos_H{i}), 2) Sac_Pos_H{i} Sac_Pos_V{i}];
    if(exist('CurveMetrics.m', 'file'))
        [metrics] = CurveMetrics(samples, Params.samplingFreq);
    else
        metrics = NaN(15, 1);
    end
    Sac_InitDir(i) = metrics(4);        % Initial Deviation
    Sac_InitAvgDev(i) = metrics(5);     % Initial Average Deviation
    Sac_MaxRawDev(i) = metrics(6);      % Maximum Raw Deviation
    Sac_PoiMaxRawDev(i) = metrics(7);   % Point of Maximum Raw Deviation (percent)
    Sac_AreaCurv(i) = metrics(8);       % Area-based Curvature Measure
    Sac_QuadCurv(i) = metrics(9);       % Quadratic-fit Curvature Measure
    Sac_CubCurvM1(i) = metrics(10);     % Cubic-fit Curvature Measure for First Point
    Sac_PoiCubCurvM1(i) = metrics(11);  % Cubic-fit Curvature First Point (percent)
    Sac_CubCurvM2(i) = metrics(12);     % Cubic-fit Curvature Measure for Second Point
    Sac_PoiCubCurvM2(i) = metrics(13);  % Cubic-fit Curvature Second Point (percent)
    Sac_CubCurvMax(i) = metrics(14);    % Cubic-fit Curvature Measure Maximum
    Sac_PoiCubCurvMax(i) = metrics(15); % Cubic-fit Curvature Point of Maximum (percent)
    
    % Number of Velocity Local Minima
    SVel = sqrt(Sac_Vel_H{i}.^2 + Sac_Vel_V{i}.^2);
    N_localmin = 0;
    for k = 1+1:length(SVel)-1
        if(SVel(k-1) > SVel(k) && SVel(k+1) > SVel(k))
            N_localmin = N_localmin + 1;
        end
    end
    Sac_NumVelLocMin(i) = N_localmin;
    
    % Peak Velocity H, V, R (deg/s)
    Sac_PkVel_H(i) = max(abs(Sac_Vel_H{i}));
    Sac_PkVel_V(i) = max(abs(Sac_Vel_V{i}));
    Sac_PkVel_R(i) = max(sqrt(Sac_Vel_H{i}.^2 + Sac_Vel_V{i}.^2));
    
    % Mean Velocity H, V, R (deg/s)
    Sac_MnVel_H(i) = Sac_Amp_H(i)/(Sac_Dur(i)/1000);
    Sac_MnVel_V(i) = Sac_Amp_V(i)/(Sac_Dur(i)/1000);
    Sac_MnVel_R(i) = sqrt(Sac_MnVel_H(i).^2 + Sac_MnVel_V(i).^2);
    
    % Velocity Profile Mean H, V, R (deg/s)
    Sac_VelProfMn_H(i) = mean(abs(Sac_Vel_H{i}));
    Sac_VelProfMn_V(i) = mean(abs(Sac_Vel_V{i}));
    Sac_VelProfMn_R(i) = mean(sqrt(Sac_Vel_H{i}.^2 + Sac_Vel_V{i}.^2));
        
    % Velocity Profile Median H, V, R (deg/s)
    Sac_VelProfMd_H(i) = median(abs(Sac_Vel_H{i}));
    Sac_VelProfMd_V(i) = median(abs(Sac_Vel_V{i}));
    Sac_VelProfMd_R(i) = median(sqrt(Sac_Vel_H{i}.^2 + Sac_Vel_V{i}.^2));
        
    % Velocity Profile Std H, V, R (deg/s)
    Sac_VelProfSd_H(i) = std(abs(Sac_Vel_H{i}));
    Sac_VelProfSd_V(i) = std(abs(Sac_Vel_V{i}));
    Sac_VelProfSd_R(i) = std(sqrt(Sac_Vel_H{i}.^2 + Sac_Vel_V{i}.^2));
        
    % Velocity Profile Skewness H, V, R (deg/s)
    Sac_VelProfSk_H(i) = skewness(abs(Sac_Vel_H{i}));
    Sac_VelProfSk_V(i) = skewness(abs(Sac_Vel_V{i}));
    Sac_VelProfSk_R(i) = skewness(sqrt(Sac_Vel_H{i}.^2 + Sac_Vel_V{i}.^2));
        
    % Velocity Profile Kurtosis H, V, R (deg/s)
    Sac_VelProfKu_H(i) = kurtosis(abs(Sac_Vel_H{i}));
    Sac_VelProfKu_V(i) = kurtosis(abs(Sac_Vel_V{i}));
    Sac_VelProfKu_R(i) = kurtosis(sqrt(Sac_Vel_H{i}.^2 + Sac_Vel_V{i}.^2));
    
    % Find limit of acceleration-deceleration phases (via peak velocity for less effect of noise)
    vel = sqrt(Sac_Vel_H{i}.^2 + Sac_Vel_V{i}.^2);
    pIdx = find(vel == max(vel));  pIdx = pIdx(1);
    
    % Peak Acceleration H, V, R (deg/s^2)
    Sac_PkAcc_H(i) = max(abs(Sac_Acc_H{i}(1:pIdx-1)));
    Sac_PkAcc_V(i) = max(abs(Sac_Acc_V{i}(1:pIdx-1)));
    Sac_PkAcc_R(i) = max(sqrt(Sac_Acc_H{i}(1:pIdx-1).^2 + Sac_Acc_V{i}(1:pIdx-1).^2));
    
    % Peak Deceleration H, V, R (deg/s^2)
    Sac_PkDec_H(i) = max(abs(Sac_Acc_H{i}(pIdx+1:end)));
    Sac_PkDec_V(i) = max(abs(Sac_Acc_V{i}(pIdx+1:end)));
    Sac_PkDec_R(i) = max(sqrt(Sac_Acc_H{i}(pIdx+1:end).^2 + Sac_Acc_V{i}(pIdx+1:end).^2));
    
    % Acceleration Profile Mean H, V, R (deg/s^2)
    Sac_AccProfMn_H(i) = mean(abs(Sac_Acc_H{i}));
    Sac_AccProfMn_V(i) = mean(abs(Sac_Acc_V{i}));
    Sac_AccProfMn_R(i) = mean(sqrt(Sac_Acc_H{i}.^2 + Sac_Acc_V{i}.^2));
        
    % Acceleration Profile Median H, V, R (deg/s^2)
    Sac_AccProfMd_H(i) = median(abs(Sac_Acc_H{i}));
    Sac_AccProfMd_V(i) = median(abs(Sac_Acc_V{i}));
    Sac_AccProfMd_R(i) = median(sqrt(Sac_Acc_H{i}.^2 + Sac_Acc_V{i}.^2));
        
    % Acceleration Profile Std H, V, R (deg/s^2)
    Sac_AccProfSd_H(i) = std(abs(Sac_Acc_H{i}));
    Sac_AccProfSd_V(i) = std(abs(Sac_Acc_V{i}));
    Sac_AccProfSd_R(i) = std(sqrt(Sac_Acc_H{i}.^2 + Sac_Acc_V{i}.^2));
        
    % Acceleration Profile Skewness H, V, R (deg/s^2)
    Sac_AccProfSk_H(i) = skewness(abs(Sac_Acc_H{i}));
    Sac_AccProfSk_V(i) = skewness(abs(Sac_Acc_V{i}));
    Sac_AccProfSk_R(i) = skewness(sqrt(Sac_Acc_H{i}.^2 + Sac_Acc_V{i}.^2));
        
    % Acceleration Profile Kurtosis H, V, R (deg/s^2)
    Sac_AccProfKu_H(i) = kurtosis(abs(Sac_Acc_H{i}));
    Sac_AccProfKu_V(i) = kurtosis(abs(Sac_Acc_V{i}));
    Sac_AccProfKu_R(i) = kurtosis(sqrt(Sac_Acc_H{i}.^2 + Sac_Acc_V{i}.^2));
    
    % Amplitude-Duration Ratio H, V, R (deg/s)
    Sac_AmpDur_Ratio_H(i) = Sac_Amp_H(i)/(0.001*Sac_Dur(i));
    if(isinf(Sac_AmpDur_Ratio_H(i))), Sac_AmpDur_Ratio_H(i) = NaN; end;
    Sac_AmpDur_Ratio_V(i) = Sac_Amp_V(i)/(0.001*Sac_Dur(i));
    if(isinf(Sac_AmpDur_Ratio_V(i))), Sac_AmpDur_Ratio_V(i) = NaN; end;
    Sac_AmpDur_Ratio_R(i) = Sac_Amp_R(i)/(0.001*Sac_Dur(i));
    if(isinf(Sac_AmpDur_Ratio_R(i))), Sac_AmpDur_Ratio_R(i) = NaN; end;
        
    % Peak Velocity-Amplitude Ratio H, V, R (deg/s/deg)
    Sac_PkVelAmp_Ratio_H(i) = Sac_PkVel_H(i)/Sac_Amp_H(i);
    if(isinf(Sac_PkVelAmp_Ratio_H(i))), Sac_PkVelAmp_Ratio_H(i) = NaN; end;
    Sac_PkVelAmp_Ratio_V(i) = Sac_PkVel_V(i)/Sac_Amp_V(i);
    if(isinf(Sac_PkVelAmp_Ratio_V(i))), Sac_PkVelAmp_Ratio_V(i) = NaN; end;
    Sac_PkVelAmp_Ratio_R(i) = Sac_PkVel_R(i)/Sac_Amp_R(i);
    if(isinf(Sac_PkVelAmp_Ratio_R(i))), Sac_PkVelAmp_Ratio_R(i) = NaN; end;
        
    % Peak Velocity-Duration Ratio (a.k.a. 'Saccadic Ratio') H, V, R (deg/s^2)
    Sac_PkVelDur_Ratio_H(i) = Sac_PkVel_H(i)/(0.001*Sac_Dur(i));
    if(isinf(Sac_PkVelDur_Ratio_H(i))), Sac_PkVelDur_Ratio_H(i) = NaN; end;
    Sac_PkVelDur_Ratio_V(i) = Sac_PkVel_V(i)/(0.001*Sac_Dur(i));
    if(isinf(Sac_PkVelDur_Ratio_V(i))), Sac_PkVelDur_Ratio_V(i) = NaN; end;
    Sac_PkVelDur_Ratio_R(i) = Sac_PkVel_R(i)/(0.001*Sac_Dur(i));
    if(isinf(Sac_PkVelDur_Ratio_R(i))), Sac_PkVelDur_Ratio_R(i) = NaN; end;
         
    % Peak Velocity-Mean Velocity Ratio (a.k.a. 'Q-Ratio') H, V, R
    Sac_PkVelMnVel_Ratio_H(i) = Sac_PkVel_H(i)/Sac_MnVel_H(i);
    if(isinf(Sac_PkVelMnVel_Ratio_H(i))), Sac_PkVelMnVel_Ratio_H(i) = NaN; end;
    Sac_PkVelMnVel_Ratio_V(i) = Sac_PkVel_V(i)/Sac_MnVel_V(i);
    if(isinf(Sac_PkVelMnVel_Ratio_V(i))), Sac_PkVelMnVel_Ratio_V(i) = NaN; end;
    Sac_PkVelMnVel_Ratio_R(i) = Sac_PkVel_R(i)/Sac_MnVel_R(i);
    if(isinf(Sac_PkVelMnVel_Ratio_R(i))), Sac_PkVelMnVel_Ratio_R(i) = NaN; end;
         
    % Peak Velocity-Local Noise Ratio R        
    v = sqrt(ET.Vectors.velX.^2 + ET.Vectors.velY.^2);
    window_start_idx = round(ET.Scalars.Data.SaccadeInfo(i).start * [Params.samplingFreq]);
    window_end_idx = max(1, ceil(window_start_idx - [Params.minFixDur]*[Params.samplingFreq]));
    VelLocNoise = v(window_start_idx:-1:window_end_idx);
    VelLocNoise = mean(VelLocNoise) + 3*std(VelLocNoise);
    Sac_PkVelLocNoise_Ratio_R(i) = Sac_PkVel_R(i)/VelLocNoise;
    if(isinf(Sac_PkVelLocNoise_Ratio_R(i))), Sac_PkVelLocNoise_Ratio_R(i) = NaN; end;
       
    % Acceleration-Deceleration Duration Ratio
    Sac_AccDecDur_Ratio(i) = length((1:pIdx-1))/length((pIdx+1:length(vel)));
    if(isinf(Sac_AccDecDur_Ratio(i))), Sac_AccDecDur_Ratio(i) = NaN; end;
       
    % Peak Acceleration-Peak Deceleration Ratio H, V, R
    Sac_PkAccPkDec_Ratio_H(i) = Sac_PkAcc_H(i)/Sac_PkDec_H(i);
    if(isinf(Sac_PkAccPkDec_Ratio_H(i))), Sac_PkAccPkDec_Ratio_H(i) = NaN; end;
    Sac_PkAccPkDec_Ratio_V(i) = Sac_PkAcc_V(i)/Sac_PkDec_V(i);
    if(isinf(Sac_PkAccPkDec_Ratio_V(i))), Sac_PkAccPkDec_Ratio_V(i) = NaN; end;
    Sac_PkAccPkDec_Ratio_R(i) = Sac_PkAcc_R(i)/Sac_PkDec_R(i);
    if(isinf(Sac_PkAccPkDec_Ratio_R(i))), Sac_PkAccPkDec_Ratio_R(i) = NaN; end;
               
end

% Create indices to filter out saccades
LargeAmpIdx = false(Sac_Num, 1);
LargeDurIdx = false(Sac_Num, 1);
for i = 1:Sac_Num
    
    if(Sac_Amp_R(i) > Params.maxSaccadeAmp), LargeAmpIdx(i) = true; end;
    if(Sac_Dur(i) > (Params.maxSaccadeDur*1000)), LargeDurIdx(i) = true; end;

end

sIdx = ~(LargeAmpIdx | LargeDurIdx);

% Calculate final saccade features (modeling, descriptive statistics)

% Saccade Rate
currFeature = length(find(sIdx))/(ET.Scalars.Data.Length/Params.samplingFreq);
features = [features, currFeature];
headers{1, 1} = 'Sac_Rate';

% Saccade Duration
currFeature = Sac_Dur(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_Dur')];

% Saccade Amplitude H, V, R
currFeature = Sac_Amp_H(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_Amp_H')];
currFeature = Sac_Amp_V(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_Amp_V')];
currFeature = Sac_Amp_R(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_Amp_R')];

% Saccade Travel Distance
currFeature = Sac_TravDist(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_TravDist')];

% Saccade Efficiency
currFeature = Sac_Efficiency(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_Efficiency')];

% Saccade Tail Efficiency
currFeature = Sac_TailEfficiency(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_TailEfficiency')];

% Saccade Tail Percent Inconsistent
currFeature = Sac_TailPrInConsist(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_TailPrInConsist')];

% Saccade Initial Direction
currFeature = Sac_InitDir(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_InitDir')];

% Saccade Initial Average Deviation
currFeature = Sac_InitAvgDev(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_InitAvgDev')];

% Saccade Maximum Raw Deviation
currFeature = Sac_MaxRawDev(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_MaxRawDev')];

% Saccade Point of Maximum Raw Deviation
currFeature = Sac_PoiMaxRawDev(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_PoiMaxRawDev')];
 
% Saccade Area-based Curvature Measure
currFeature = Sac_AreaCurv(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_AreaCurv')];

% Saccade Quadratic-fit Curvature Measure
currFeature = Sac_QuadCurv(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_QuadCurv')];

% Saccade Cubic-fit Curvature Measure for First Point
currFeature = Sac_CubCurvM1(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_CubCurvM1')];

% Saccade Cubic-fit Curvature First Point
currFeature = Sac_PoiCubCurvM1(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_PoiCubCurvM1')];     

% Saccade Cubic-fit Curvature Measure for Second Point
currFeature = Sac_CubCurvM2(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_CubCurvM2')];

% Saccade Cubic-fit Curvature Second Point
currFeature = Sac_PoiCubCurvM2(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_PoiCubCurvM2')];

% Saccade Cubic-fit Curvature Measure Maximum
currFeature = Sac_CubCurvMax(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_CubCurvMax')];

% Saccade Cubic-fit Curvature Point of Maximum
currFeature = Sac_PoiCubCurvMax(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_PoiCubCurvMax')];

% Saccade Number of Velocity Local Minima
currFeature = Sac_NumVelLocMin(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_NumVelLocMin')];

% Saccade Peak Velocity H, V, R
currFeature = Sac_PkVel_H(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_PkVel_H')];
currFeature = Sac_PkVel_V(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_PkVel_V')];
currFeature = Sac_PkVel_R(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_PkVel_R')];

% Saccade Velocity Profile Mean H, V, R
currFeature = Sac_VelProfMn_H(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_VelProfMn_H')];
currFeature = Sac_VelProfMn_V(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_VelProfMn_V')];
currFeature = Sac_VelProfMn_R(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_VelProfMn_R')];

% Saccade Velocity Profile Median H, V, R
currFeature = Sac_VelProfMd_H(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_VelProfMd_H')];
currFeature = Sac_VelProfMd_V(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_VelProfMd_V')];
currFeature = Sac_VelProfMd_R(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_VelProfMd_R')];

% Saccade Velocity Profile Std H, V, R
currFeature = Sac_VelProfSd_H(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_VelProfSd_H')];
currFeature = Sac_VelProfSd_V(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_VelProfSd_V')];
currFeature = Sac_VelProfSd_R(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_VelProfSd_R')];

% Saccade Velocity Profile Skewness H, V, R
currFeature = Sac_VelProfSk_H(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_VelProfSk_H')];
currFeature = Sac_VelProfSk_V(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_VelProfSk_V')];
currFeature = Sac_VelProfSk_R(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_VelProfSk_R')];

% Saccade Velocity Profile Kurtosis H, V, R
currFeature = Sac_VelProfKu_H(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_VelProfKu_H')];
currFeature = Sac_VelProfKu_V(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_VelProfKu_V')];
currFeature = Sac_VelProfKu_R(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_VelProfKu_R')];

% Saccade Peak Acceleration H, V, R
currFeature = Sac_PkAcc_H(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_PkAcc_H')];
currFeature = Sac_PkAcc_V(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_PkAcc_V')];
currFeature = Sac_PkAcc_R(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_PkAcc_R')];

% Saccade Peak Deceleration H, V, R
currFeature = Sac_PkDec_H(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_PkDec_H')];
currFeature = Sac_PkDec_V(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_PkDec_V')];
currFeature = Sac_PkDec_R(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_PkDec_R')];

% Saccade Acceleration Profile Mean H, V, R
currFeature = Sac_AccProfMn_H(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_AccProfMn_H')];
currFeature = Sac_AccProfMn_V(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_AccProfMn_V')];
currFeature = Sac_AccProfMn_R(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_AccProfMn_R')];

% Saccade Acceleration Profile Median H, V, R
currFeature = Sac_AccProfMd_H(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_AccProfMd_H')];
currFeature = Sac_AccProfMd_V(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_AccProfMd_V')];
currFeature = Sac_AccProfMd_R(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_AccProfMd_R')];

% Saccade Acceleration Profile Std H, V, R
currFeature = Sac_AccProfSd_H(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_AccProfSd_H')];
currFeature = Sac_AccProfSd_V(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_AccProfSd_V')];
currFeature = Sac_AccProfSd_R(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_AccProfSd_R')];

% Saccade Acceleration Profile Skewness H, V, R
currFeature = Sac_AccProfSk_H(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_AccProfSk_H')];
currFeature = Sac_AccProfSk_V(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_AccProfSk_V')];
currFeature = Sac_AccProfSk_R(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_AccProfSk_R')];

% Saccade Acceleration Profile Kurtosis H, V, R
currFeature = Sac_AccProfKu_H(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_AccProfKu_H')];
currFeature = Sac_AccProfKu_V(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_AccProfKu_V')];
currFeature = Sac_AccProfKu_R(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_AccProfKu_R')];

% Saccade Amplitude-Duration Ratio H, V, R
currFeature = Sac_AmpDur_Ratio_H(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_AmpDur_Ratio_H')];
currFeature = Sac_AmpDur_Ratio_V(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_AmpDur_Ratio_V')];
currFeature = Sac_AmpDur_Ratio_R(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_AmpDur_Ratio_R')];

% Saccade Peak Velocity-Amplitude Ratio H, V, R
currFeature = Sac_PkVelAmp_Ratio_H(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_PkVelAmp_Ratio_H')];
currFeature = Sac_PkVelAmp_Ratio_V(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_PkVelAmp_Ratio_V')];
currFeature = Sac_PkVelAmp_Ratio_R(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_PkVelAmp_Ratio_R')];

% Saccade Peak Velocity-Duration Ratio H, V, R
currFeature = Sac_PkVelDur_Ratio_H(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_PkVelDur_Ratio_H')];
currFeature = Sac_PkVelDur_Ratio_V(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_PkVelDur_Ratio_V')];
currFeature = Sac_PkVelDur_Ratio_R(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_PkVelDur_Ratio_R')];

% Saccade Peak Velocity-Mean Velocity Ratio H, V, R
currFeature = Sac_PkVelMnVel_Ratio_H(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_PkVelMnVel_Ratio_H')];
currFeature = Sac_PkVelMnVel_Ratio_V(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_PkVelMnVel_Ratio_V')];
currFeature = Sac_PkVelMnVel_Ratio_R(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_PkVelMnVel_Ratio_R')];

% Saccade Peak Velocity-Local Noise Ratio
currFeature = Sac_PkVelLocNoise_Ratio_R(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_PkVelLocNoise_Ratio_R')];

% Saccade Acceleration-Deceleration Duration Ratio
currFeature = Sac_AccDecDur_Ratio(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_AccDecDur_Ratio')];

% Saccade Peak Acceleration-Peak Deceleration Ratio H, V, R
currFeature = Sac_PkAccPkDec_Ratio_H(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_PkAccPkDec_Ratio_H')];
currFeature = Sac_PkAccPkDec_Ratio_V(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_PkAccPkDec_Ratio_V')];
currFeature = Sac_PkAccPkDec_Ratio_R(sIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('Sac_PkAccPkDec_Ratio_R')];

% Saccade 'Main Sequence' Amp-Dur Features (Intercept, Slope, R^2)
X = Sac_Dur(sIdx); Y = Sac_Amp_R(sIdx); 
XX = [ones(size(X)) X];
[b, ~, ~, ~, stats] = regress(Y, XX); 
currFeature = b(1);
features = [features, currFeature];
headers = [headers, 'Sac_AmpDurFitLn_Intercept_R'];
currFeature = b(2);
features = [features, currFeature];
headers = [headers, 'Sac_AmpDurFitLn_Slope_R'];
currFeature = stats(1);
features = [features, currFeature];
headers = [headers, 'Sac_AmpDurFitLn_R2_R'];

% Saccade 'Main Sequence' PkVel-Amp Features (Intercept, Slope, R^2)
X = log10(Sac_Amp_R(sIdx)); Y = log10(Sac_PkVel_R(sIdx));
XX = [ones(size(X)) X]; 
[b, ~, ~, ~, stats] = regress(Y, XX); 
currFeature = b(1);
features = [features, currFeature];
headers = [headers, 'Sac_PkVelAmpFitLn_Intercept_R'];
currFeature = b(2);
features = [features, currFeature];
headers = [headers, 'Sac_PkVelAmpFitLn_Slope_R'];
currFeature = stats(1);
features = [features, currFeature];
headers = [headers, 'Sac_PkVelAmpFitLn_R2_R'];

% Reading Behavior Features: Number of Rightward Small Saccades/sec
Sac_SmRight_Rate = length(find(Sac_Direction_H(sIdx) == 1))/(ET.Scalars.Data.Length/Params.samplingFreq);
currFeature = Sac_SmRight_Rate;
features = [features, currFeature];
headers = [headers, 'Sac_SmRight_Rate'];

% Reading Behavior Features: Number of Leftward Small Saccades/sec
Sac_SmLeft_Rate = length(find(Sac_Direction_H(sIdx) == -1))/(ET.Scalars.Data.Length/Params.samplingFreq);
currFeature = Sac_SmLeft_Rate;
features = [features, currFeature];
headers = [headers, 'Sac_SmLeft_Rate'];

% Reading Behavior Features: Number of Rightward Large Saccades/sec
Sac_LgRight_Rate = length(find(Sac_Direction_H(LargeAmpIdx) == 1))/(ET.Scalars.Data.Length/Params.samplingFreq);
currFeature = Sac_LgRight_Rate;
features = [features, currFeature];
headers = [headers, 'Sac_LgRight_Rate'];

% Reading Behavior Features: Number of Leftward Large Saccades/sec
Sac_LgLeft_Rate = length(find(Sac_Direction_H(LargeAmpIdx) == -1))/(ET.Scalars.Data.Length/Params.samplingFreq);
currFeature = Sac_LgLeft_Rate;
features = [features, currFeature];
headers = [headers, 'Sac_LgLeft_Rate'];

% Reading Behavior Features: Ratio of Leftward to Rightward Small Saccades
Sac_SmLeftSmRight_Ratio = Sac_SmLeft_Rate/Sac_SmRight_Rate;
currFeature = Sac_SmLeftSmRight_Ratio;
features = [features, currFeature];
headers = [headers, 'Sac_SmLeftSmRight_Ratio'];

% Reading Behavior Features: Ratio of All Small Saccades to Leftward Large Saccades
Sac_SmAllLgLeft_Ratio = (Sac_SmLeft_Rate + Sac_SmRight_Rate)/Sac_LgLeft_Rate;
currFeature = Sac_SmAllLgLeft_Ratio;
features = [features, currFeature];
headers = [headers, 'Sac_SmAllLgLeft_Ratio'];




% ----------------------------------------
% Post-Saccadic Oscillation (PSO) Features 
% ----------------------------------------

% Get number of saccades (we use this to initialize vectors so that PSOs 
% are 'synchronized' with their preceding saccades)
Sac_Num = ET.Scalars.Data.N_Saccades;
if(length(ET.Scalars.Data.PSOInfo) < length(ET.Scalars.Data.SaccadeInfo))
    Sac_Num = length(ET.Scalars.Data.PSOInfo);
end

% Extract PSO position, velocity, and acceleration profiles
PSO_Start = zeros(Sac_Num, 1);
PSO_End = zeros(Sac_Num, 1);
PSO_Pos_H = cell(Sac_Num, 1);
PSO_Pos_V = cell(Sac_Num, 1);
PSO_Vel_H = cell(Sac_Num, 1);
PSO_Vel_V = cell(Sac_Num, 1);
PSO_Acc_H = cell(Sac_Num, 1);
PSO_Acc_V = cell(Sac_Num, 1);

for i = 1:Sac_Num
    
    if(ET.Scalars.Data.PSOInfo(i).end == 0), continue, end;
    % Convert from time to samples
    PSO_Start(i) = round(ET.Scalars.Data.PSOInfo(i).start * Params.samplingFreq);
    PSO_End(i) = round(ET.Scalars.Data.PSOInfo(i).end * Params.samplingFreq);
    % Extract profiles
    PSO_Pos_H{i} = ET.Vectors.Xsmo(PSO_Start(i):PSO_End(i));
    PSO_Pos_V{i} = ET.Vectors.Ysmo(PSO_Start(i):PSO_End(i));
    PSO_Vel_H{i} = ET.Vectors.velX(PSO_Start(i):PSO_End(i));
    PSO_Vel_V{i} = ET.Vectors.velY(PSO_Start(i):PSO_End(i));
    PSO_Acc_H{i} = ET.Vectors.accX(PSO_Start(i):PSO_End(i));
    PSO_Acc_V{i} = ET.Vectors.accY(PSO_Start(i):PSO_End(i));
    
end

% Calculate feature values from multiple PSO instances
PSO_Dur = zeros(Sac_Num, 1);
PSO_Interv = zeros(Sac_Num, 1);
PSO_MaxAbsDev_H = zeros(Sac_Num, 1);
PSO_MaxAbsDev_V = zeros(Sac_Num, 1);
PSO_MaxAbsDev_R = zeros(Sac_Num, 1);
PSO_NumPosVlls_H = zeros(Sac_Num, 1);
PSO_NumPosVlls_V = zeros(Sac_Num, 1);
PSO_NumPosVlls_R = zeros(Sac_Num, 1);
PSO_NumPosPks_H = zeros(Sac_Num, 1);
PSO_NumPosPks_V = zeros(Sac_Num, 1);
PSO_NumPosPks_R = zeros(Sac_Num, 1);
PSO_PkVel_H = zeros(Sac_Num, 1);
PSO_PkVel_V = zeros(Sac_Num, 1);
PSO_PkVel_R = zeros(Sac_Num, 1);
PSO_VelProfMn_H = zeros(Sac_Num, 1);
PSO_VelProfMn_V = zeros(Sac_Num, 1);
PSO_VelProfMn_R = zeros(Sac_Num, 1);
PSO_VelProfMd_H = zeros(Sac_Num, 1);
PSO_VelProfMd_V = zeros(Sac_Num, 1);
PSO_VelProfMd_R = zeros(Sac_Num, 1);
PSO_VelProfSd_H = zeros(Sac_Num, 1);
PSO_VelProfSd_V = zeros(Sac_Num, 1);
PSO_VelProfSd_R = zeros(Sac_Num, 1);
PSO_VelProfSk_H = zeros(Sac_Num, 1);
PSO_VelProfSk_V = zeros(Sac_Num, 1);
PSO_VelProfSk_R = zeros(Sac_Num, 1);
PSO_VelProfKu_H = zeros(Sac_Num, 1);
PSO_VelProfKu_V = zeros(Sac_Num, 1);
PSO_VelProfKu_R = zeros(Sac_Num, 1);
PSO_AccProfMn_H = zeros(Sac_Num, 1);
PSO_AccProfMn_V = zeros(Sac_Num, 1);
PSO_AccProfMn_R = zeros(Sac_Num, 1);
PSO_AccProfMd_H = zeros(Sac_Num, 1);
PSO_AccProfMd_V = zeros(Sac_Num, 1);
PSO_AccProfMd_R = zeros(Sac_Num, 1);
PSO_AccProfSd_H = zeros(Sac_Num, 1);
PSO_AccProfSd_V = zeros(Sac_Num, 1);
PSO_AccProfSd_R = zeros(Sac_Num, 1);
PSO_AccProfSk_H = zeros(Sac_Num, 1);
PSO_AccProfSk_V = zeros(Sac_Num, 1);
PSO_AccProfSk_R = zeros(Sac_Num, 1);
PSO_AccProfKu_H = zeros(Sac_Num, 1);
PSO_AccProfKu_V = zeros(Sac_Num, 1);
PSO_AccProfKu_R = zeros(Sac_Num, 1);
PSO_SacDurPSODur_Ratio = zeros(Sac_Num, 1);
PSO_SacAmpPSODur_Ratio_H = zeros(Sac_Num, 1);
PSO_SacAmpPSODur_Ratio_V = zeros(Sac_Num, 1);
PSO_SacAmpPSODur_Ratio_R = zeros(Sac_Num, 1);
PSO_SacAmpPSOMaxAbsDev_Ratio_H = zeros(Sac_Num, 1);
PSO_SacAmpPSOMaxAbsDev_Ratio_V = zeros(Sac_Num, 1);
PSO_SacAmpPSOMaxAbsDev_Ratio_R = zeros(Sac_Num, 1);
PSO_SacPkVelPSOPkVel_Ratio_H = zeros(Sac_Num, 1);
PSO_SacPkVelPSOPkVel_Ratio_V = zeros(Sac_Num, 1);
PSO_SacPkVelPSOPkVel_Ratio_R = zeros(Sac_Num, 1);

newPSOEnd = 0;
for i = 1:Sac_Num
    
    % Duration (ms)
    PSO_Dur(i) = 1000*(PSO_End(i) - PSO_Start(i))/[Params.samplingFreq];
        
    % Inter-PSO Interval (ms)
    if(ET.Scalars.Data.PSOInfo(i).end ~= 0 || isempty(ET.Scalars.Data.PSOInfo(i).end))
        PSO_Interv(i) = 1000*(PSO_Start(i) - newPSOEnd)/[Params.samplingFreq];
        newPSOEnd = PSO_End(i);
    end
    
    % Maximum absolute deviation H, V, R (deg)
    if(isempty(abs(max(PSO_Pos_H{i}) - min(PSO_Pos_H{i}))))
        PSO_MaxAbsDev_H(i) = NaN;
    else
        PSO_MaxAbsDev_H(i) = abs(max(PSO_Pos_H{i}) - min(PSO_Pos_H{i}));
    end
    
    if(isempty(abs(max(PSO_Pos_V{i}) - min(PSO_Pos_V{i}))))
        PSO_MaxAbsDev_V(i) = NaN;
    else
        PSO_MaxAbsDev_V(i) = abs(max(PSO_Pos_V{i}) - min(PSO_Pos_V{i}));
    end
    
    if(isempty(max(sqrt(PSO_Pos_H{i}.^2 + PSO_Pos_V{i}.^2)) - min(sqrt(PSO_Pos_H{i}.^2 + PSO_Pos_V{i}.^2))))
        PSO_MaxAbsDev_R(i) = NaN;
    else
        PSO_MaxAbsDev_R(i) = max(sqrt(PSO_Pos_H{i}.^2 + PSO_Pos_V{i}.^2)) - min(sqrt(PSO_Pos_H{i}.^2 + PSO_Pos_V{i}.^2));
    end
    
    % Number of Local Position Valleys H, V, R
    PSO_NumPosVlls_H(i) = length(find(diff(sign(diff(PSO_Pos_H{i}))) > 0));
    PSO_NumPosVlls_V(i) = length(find(diff(sign(diff(PSO_Pos_V{i}))) > 0));
    PSO_NumPosVlls_R(i) = length(find(diff(sign(diff(sqrt(PSO_Pos_H{i}.^2 + PSO_Pos_V{i}.^2)))) > 0));
        
    % Number of Local Position Peaks H, V, R
    PSO_NumPosPks_H(i) = length(find(diff(sign(diff(PSO_Pos_H{i}))) < 0));
    PSO_NumPosPks_V(i) = length(find(diff(sign(diff(PSO_Pos_V{i}))) < 0));
    PSO_NumPosPks_R(i) = length(find(diff(sign(diff(sqrt(PSO_Pos_H{i}.^2 + PSO_Pos_V{i}.^2)))) < 0));
        
    % Peak Velocity H, V, R (deg/s)
    if(isempty(max(abs(PSO_Vel_H{i}))))
        PSO_PkVel_H(i) = NaN;
    else
        PSO_PkVel_H(i) = max(abs(PSO_Vel_H{i}));
    end
    
    if(isempty(max(abs(PSO_Vel_V{i}))))
        PSO_PkVel_V(i) = NaN;
    else
        PSO_PkVel_V(i) = max(abs(PSO_Vel_V{i}));
    end
    
    if(isempty(max(sqrt(PSO_Vel_H{i}.^2 + PSO_Vel_V{i}.^2))))
        PSO_PkVel_R(i) = NaN;
    else
        PSO_PkVel_R(i) = max(sqrt(PSO_Vel_H{i}.^2 + PSO_Vel_V{i}.^2));
    end
    
    % Velocity Profile Mean H, V, R (deg/s)
    PSO_VelProfMn_H(i) = mean(abs(PSO_Vel_H{i}));
    PSO_VelProfMn_V(i) = mean(abs(PSO_Vel_V{i}));
    PSO_VelProfMn_R(i) = mean(sqrt(PSO_Vel_H{i}.^2 + PSO_Vel_V{i}.^2));
        
    % Velocity Profile Median H, V, R (deg/s)
    PSO_VelProfMd_H(i) = median(abs(PSO_Vel_H{i}));
    PSO_VelProfMd_V(i) = median(abs(PSO_Vel_V{i}));
    PSO_VelProfMd_R(i) = median(sqrt(PSO_Vel_H{i}.^2 + PSO_Vel_V{i}.^2));
        
    % Velocity Profile Std H, V, R (deg/s)
    PSO_VelProfSd_H(i) = std(abs(PSO_Vel_H{i}));
    PSO_VelProfSd_V(i) = std(abs(PSO_Vel_V{i}));
    PSO_VelProfSd_R(i) = std(sqrt(PSO_Vel_H{i}.^2 + PSO_Vel_V{i}.^2));
        
    % Velocity Profile Skewness H, V, R (deg/s)
    PSO_VelProfSk_H(i) = skewness(abs(PSO_Vel_H{i}));
    PSO_VelProfSk_V(i) = skewness(abs(PSO_Vel_V{i}));
    PSO_VelProfSk_R(i) = skewness(sqrt(PSO_Vel_H{i}.^2 + PSO_Vel_V{i}.^2));
        
    % Velocity Profile Kurtosis H, V, R (deg/s)
    PSO_VelProfKu_H(i) = kurtosis(abs(PSO_Vel_H{i}));
    PSO_VelProfKu_V(i) = kurtosis(abs(PSO_Vel_V{i}));
    PSO_VelProfKu_R(i) = kurtosis(sqrt(PSO_Vel_H{i}.^2 + PSO_Vel_V{i}.^2));
        
    % Acceleration Profile Mean H, V, R (deg/s^2)
    PSO_AccProfMn_H(i) = mean(abs(PSO_Acc_H{i}));
    PSO_AccProfMn_V(i) = mean(abs(PSO_Acc_V{i}));
    PSO_AccProfMn_R(i) = mean(sqrt(PSO_Acc_H{i}.^2 + PSO_Acc_V{i}.^2));
        
    % Acceleration Profile Median H, V, R (deg/s^2)
    PSO_AccProfMd_H(i) = median(abs(PSO_Acc_H{i}));
    PSO_AccProfMd_V(i) = median(abs(PSO_Acc_V{i}));
    PSO_AccProfMd_R(i) = median(sqrt(PSO_Acc_H{i}.^2 + PSO_Acc_V{i}.^2));
        
    % Acceleration Profile Std H, V, R (deg/s^2)
    PSO_AccProfSd_H(i) = std(abs(PSO_Acc_H{i}));
    PSO_AccProfSd_V(i) = std(abs(PSO_Acc_V{i}));
    PSO_AccProfSd_R(i) = std(sqrt(PSO_Acc_H{i}.^2 + PSO_Acc_V{i}.^2));
        
    % Acceleration Profile Skewness H, V, R (deg/s^2)
    PSO_AccProfSk_H(i) = skewness(abs(PSO_Acc_H{i}));
    PSO_AccProfSk_V(i) = skewness(abs(PSO_Acc_V{i}));
    PSO_AccProfSk_R(i) = skewness(sqrt(PSO_Acc_H{i}.^2 + PSO_Acc_V{i}.^2));
        
    % Acceleration Profile Kurtosis H, V, R (deg/s^2)
    PSO_AccProfKu_H(i) = kurtosis(abs(PSO_Acc_H{i}));
    PSO_AccProfKu_V(i) = kurtosis(abs(PSO_Acc_V{i}));
    PSO_AccProfKu_R(i) = kurtosis(sqrt(PSO_Acc_H{i}.^2 + PSO_Acc_V{i}.^2));
        
    % Saccade Duration-PSO Duration Ratio
    PSO_SacDurPSODur_Ratio(i) = Sac_Dur(i)/PSO_Dur(i);
    if(isinf(PSO_SacDurPSODur_Ratio(i))), PSO_SacDurPSODur_Ratio(i) = NaN; end;
        
    % Saccade Amplitude-PSO Duration Ratio H, V, R (deg/s)
    PSO_SacAmpPSODur_Ratio_H(i) = Sac_Amp_H(i)/(0.001*PSO_Dur(i));
    if(isinf(PSO_SacAmpPSODur_Ratio_H(i))), PSO_SacAmpPSODur_Ratio_H(i) = NaN; end; 
    PSO_SacAmpPSODur_Ratio_V(i) = Sac_Amp_V(i)/(0.001*PSO_Dur(i));
    if(isinf(PSO_SacAmpPSODur_Ratio_V(i))), PSO_SacAmpPSODur_Ratio_V(i) = NaN; end; 
    PSO_SacAmpPSODur_Ratio_R(i) = Sac_Amp_R(i)/(0.001*PSO_Dur(i));
    if(isinf(PSO_SacAmpPSODur_Ratio_R(i))), PSO_SacAmpPSODur_Ratio_R(i) = NaN; end;
        
    % Saccade Amplitude-PSO Maximum Absolute Deviation Ratio H, V, R
    PSO_SacAmpPSOMaxAbsDev_Ratio_H(i) = Sac_Amp_H(i)/PSO_MaxAbsDev_H(i);
    if(isinf(PSO_SacAmpPSOMaxAbsDev_Ratio_H(i))), PSO_SacAmpPSOMaxAbsDev_Ratio_H(i) = NaN; end; 
    PSO_SacAmpPSOMaxAbsDev_Ratio_V(i) = Sac_Amp_V(i)/PSO_MaxAbsDev_V(i);
    if(isinf(PSO_SacAmpPSOMaxAbsDev_Ratio_V(i))), PSO_SacAmpPSOMaxAbsDev_Ratio_V(i) = NaN; end; 
    PSO_SacAmpPSOMaxAbsDev_Ratio_R(i) = Sac_Amp_R(i)/PSO_MaxAbsDev_R(i);
    if(isinf(PSO_SacAmpPSOMaxAbsDev_Ratio_R(i))), PSO_SacAmpPSOMaxAbsDev_Ratio_R(i) = NaN; end;
        
    % Saccade Peak Velocity-PSO Peak Velocity Ratio H, V, R
    PSO_SacPkVelPSOPkVel_Ratio_H(i) = Sac_PkVel_H(i)/PSO_PkVel_H(i);
    if(isinf(PSO_SacPkVelPSOPkVel_Ratio_H(i))), PSO_SacPkVelPSOPkVel_Ratio_H(i) = NaN; end; 
    PSO_SacPkVelPSOPkVel_Ratio_V(i) = Sac_PkVel_V(i)/PSO_PkVel_V(i);
    if(isinf(PSO_SacPkVelPSOPkVel_Ratio_V(i))), PSO_SacPkVelPSOPkVel_Ratio_V(i) = NaN; end; 
    PSO_SacPkVelPSOPkVel_Ratio_R(i) = Sac_PkVel_R(i)/PSO_PkVel_R(i);
    if(isinf(PSO_SacPkVelPSOPkVel_Ratio_R(i))), PSO_SacPkVelPSOPkVel_Ratio_R(i) = NaN; end;

end

% Create indices to filter out PSOs
pIdx = false(Sac_Num, 1);
pSl = 0;
pMo = 0;
pFa = 0;

for i = 1:Sac_Num
    
    if(~sIdx(i)), continue, end;  
    if(isnan(PSO_PkVel_R(i))), continue, end;
    
    pIdx(i) = true;
    
    if(PSO_PkVel_R(i) > Params.PeakVelocityThreshold)
        pFa = pFa + 1;
    elseif(PSO_PkVel_R(i) <= Params.PeakVelocityThreshold && PSO_PkVel_R(i) > Params.SaccadeSubthreshold)
        pMo = pMo + 1;
    elseif(PSO_PkVel_R(i) <= Params.SaccadeSubthreshold && PSO_PkVel_R(i) > Params.SmallPSOVelocityThreshold)
        pSl = pSl + 1;     
    end
    
end

% Calculate final PSO features (modeling, descriptive statistics)

% PSO Duration
currFeature = PSO_Dur(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_Dur')];

% Inter-PSO Interval
tmp_pIdx = pIdx; tmp_pIdx(1) = false; % Jump first value
currFeature = PSO_Interv(tmp_pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_Interv')];

% PSO Percentages
currFeature = 100 * (length(find(pIdx))/length(find(sIdx)));
features = [features, currFeature];
headers = [headers, 'PSO_Pr_PSOs'];
currFeature = 100 * (pSl/length(find(pIdx)));
features = [features, currFeature];
headers = [headers, 'PSO_Pr_PSOs_Sm'];
currFeature = 100 * (pMo/length(find(pIdx)));
features = [features, currFeature];
headers = [headers, 'PSO_Pr_PSOs_Mo'];
currFeature = 100 * (pFa/length(find(pIdx)));
features = [features, currFeature];
headers = [headers, 'PSO_Pr_PSOs_Lg'];

% PSO Max Absolute Deviation H, V, R
currFeature = PSO_MaxAbsDev_H(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_MaxAbsDev_H')];
currFeature = PSO_MaxAbsDev_V(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_MaxAbsDev_V')];
currFeature = PSO_MaxAbsDev_R(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_MaxAbsDev_R')];

% PSO Number of Valleys H, V, R
currFeature = PSO_NumPosVlls_H(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_NumPosVlls_H')];
currFeature = PSO_NumPosVlls_V(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_NumPosVlls_V')];
currFeature = PSO_NumPosVlls_R(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_NumPosVlls_R')];

% PSO Number of Peaks H, V, R
currFeature = PSO_NumPosPks_H(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_NumPosPks_H')];
currFeature = PSO_NumPosPks_V(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_NumPosPks_V')];
currFeature = PSO_NumPosPks_R(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_NumPosPks_R')];

% PSO Peak Velocity H, V, R
currFeature = PSO_PkVel_H(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_PkVel_H')];
currFeature = PSO_PkVel_V(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_PkVel_V')];
currFeature = PSO_PkVel_R(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_PkVel_R')];

% PSO Velocity Profile Mean H, V, R
currFeature = PSO_VelProfMn_H(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_VelProfMn_H')];
currFeature = PSO_VelProfMn_V(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_VelProfMn_V')];
currFeature = PSO_VelProfMn_R(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_VelProfMn_R')];

% PSO Velocity Profile Median H, V, R
currFeature = PSO_VelProfMd_H(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_VelProfMd_H')];
currFeature = PSO_VelProfMd_V(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_VelProfMd_V')];
currFeature = PSO_VelProfMd_R(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_VelProfMd_R')];

% PSO Velocity Profile Std H, V, R
currFeature = PSO_VelProfSd_H(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_VelProfSd_H')];
currFeature = PSO_VelProfSd_V(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_VelProfSd_V')];
currFeature = PSO_VelProfSd_R(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_VelProfSd_R')];

% PSO Velocity Profile Skewness H, V, R
currFeature = PSO_VelProfSk_H(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_VelProfSk_H')];
currFeature = PSO_VelProfSk_V(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_VelProfSk_V')];
currFeature = PSO_VelProfSk_R(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_VelProfSk_R')];

% PSO Velocity Profile Kurtosis H, V, R
currFeature = PSO_VelProfKu_H(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_VelProfKu_H')];
currFeature = PSO_VelProfKu_V(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_VelProfKu_V')];
currFeature = PSO_VelProfKu_R(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_VelProfKu_R')];

% PSO Acceleration Profile Mean H, V, R
currFeature = PSO_AccProfMn_H(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_AccProfMn_H')];
currFeature = PSO_AccProfMn_V(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_AccProfMn_V')];
currFeature = PSO_AccProfMn_R(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_AccProfMn_R')];

% PSO Acceleration Profile Median H, V, R
currFeature = PSO_AccProfMd_H(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_AccProfMd_H')];
currFeature = PSO_AccProfMd_V(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_AccProfMd_V')];
currFeature = PSO_AccProfMd_R(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_AccProfMd_R')];

% PSO Acceleration Profile Std H, V, R
currFeature = PSO_AccProfSd_H(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_AccProfSd_H')];
currFeature = PSO_AccProfSd_V(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_AccProfSd_V')];
currFeature = PSO_AccProfSd_R(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_AccProfSd_R')];

% PSO Acceleration Profile Skewness H, V, R
currFeature = PSO_AccProfSk_H(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_AccProfSk_H')];
currFeature = PSO_AccProfSk_V(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_AccProfSk_V')];
currFeature = PSO_AccProfSk_R(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_AccProfSk_R')];

% PSO Acceleration Profile Kurtosis H, V, R
currFeature = PSO_AccProfKu_H(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_AccProfKu_H')];
currFeature = PSO_AccProfKu_V(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_AccProfKu_V')];
currFeature = PSO_AccProfKu_R(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_AccProfKu_R')];

% Saccade Duration-PSO Duration Ratio
currFeature = PSO_SacDurPSODur_Ratio(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_SacDurPSODur_Ratio')];

% Saccade Amplitude-PSO Duration Ratio H, V, R
currFeature = PSO_SacAmpPSODur_Ratio_H(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_SacAmpPSODur_Ratio_H')];
currFeature = PSO_SacAmpPSODur_Ratio_V(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_SacAmpPSODur_Ratio_V')];
currFeature = PSO_SacAmpPSODur_Ratio_R(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_SacAmpPSODur_Ratio_R')];

% Saccade Amplitude-PSO Maximum Absolute Deviation Ratio H, V, R
currFeature = PSO_SacAmpPSOMaxAbsDev_Ratio_H(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_SacAmpPSOMaxAbsDev_Ratio_H')];
currFeature = PSO_SacAmpPSOMaxAbsDev_Ratio_V(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_SacAmpPSOMaxAbsDev_Ratio_V')];
currFeature = PSO_SacAmpPSOMaxAbsDev_Ratio_R(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_SacAmpPSOMaxAbsDev_Ratio_R')];

% Saccade Peak Velocity-PSO Peak Velocity Ratio H, V, R
currFeature = PSO_SacPkVelPSOPkVel_Ratio_H(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_SacPkVelPSOPkVel_Ratio_H')];
currFeature = PSO_SacPkVelPSOPkVel_Ratio_V(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_SacPkVelPSOPkVel_Ratio_V')];
currFeature = PSO_SacPkVelPSOPkVel_Ratio_R(pIdx);
features = [features, stat_fncs(currFeature)];
headers = [headers, stat_headers('PSO_SacPkVelPSOPkVel_Ratio_R')];

end