% -------------------------------------------------------------------------
% Copyright  2017, Texas State University. All rights reserved.
% -------------------------------------------------------------------------
%
% This work is licensed under a Creative Commons Attribution-NonCommercial
% -ShareAlike 3.0 United States License. See license conditions at:
% https://creativecommons.org/licenses/by-nc-sa/3.0/us/
%
% For all other uses, please contact the Office for Commercialization and 
% Industry Relations at Texas State University http://www.txstate.edu/ocir/
%
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% Created by: Ioannis Rigas (rigas.ioann@gmail.com)
% -------------------------------------------------------------------------


% ------------------------------------------------------------------------- 
% NOTES
% -------------------------------------------------------------------------
% Description of fields of the expected "ET" structure in input files:
% -------------------------------------------------------------------------
% ET.Vectors.Xorg : vector of raw horizontal eye movement signal
% ET.Vectors.Yorg : vector of raw vertical eye movement signal
% ET.Vectors.Xsmo : vector of smoothed horizontal eye movement signal
% ET.Vectors.Ysmo : vector of smoothed vertical eye movement signal
% ET.Vectors.velX : vector of smoothed horizontal velocity signal
% ET.Vectors.velY : vector of smoothed vertical velocity signal
% ET.Vectors.accX : vector of smoothed horizontal acceleration signal 
% ET.Vectors.accY : vector of smoothed vertical acceleration signal

% ET.Scalars.Data.Length                : length of vectors (samples)
% ET.Scalars.Data.N_Fixations           : number of classified fixations 
% ET.Scalars.Data.N_Saccades            : number of classified saccades
% ET.Scalars.Data.N_PSOs                : number of classified PSOs
% ET.Scalars.Data.FixationInfo(i).start : start time of i-th fixation (s)
% ET.Scalars.Data.FixationInfo(i).end   : end time of i-th fixation (s)
% ET.Scalars.Data.SaccadeInfo(i).start  : start time of i-th saccade (s)
% ET.Scalars.Data.SaccadeInfo(i).end    : end time of i-th saccade (s)
% ET.Scalars.Data.PSOInfo(i).start      : start time of i-th pso (s)
% ET.Scalars.Data.PSOInfo(i).end        : end time of i-th pso (s)
% -------------------------------------------------------------------------
% Description of fields of the "Params" structure:
% -------------------------------------------------------------------------
% Params.samplingFreq                        : sampling frequency (Hz)
% Params.minFixDur                           : threshold for minimum 
%                                              fixation duration (s)
% Params.maxSaccadeAmp                       : threshold for maximum 
%                                              saccade amplitude (deg.)
% Params.maxSaccadeDur                       : threshold for maximum 
%                                              saccade duration (s)
% Params.ExcludeAboveThisVelForNoiseAnalysis : velocity threshold (deg./s) 
%                                              for exclussion of samples in 
%                                              overall fixation statistics
% Params.PeakVelocityThreshold               : velocity threshold (deg./s) 
%                                              used during estimation of 
%                                              "fast" and "moderate" PSOs
% Params.SaccadeSubthreshold                 : velocity threshold (deg./s) 
%                                              used during estimation of 
%                                              "moderate" and "slow" PSOs
% Params.SmallPSOVelocityThreshold           : velocity threshold (deg./s)
%                                              used during estimation of 
%                                              "slow" PSOs
% -------------------------------------------------------------------------


close all; clear; clc;

% Specify directories of input data and output data 
Input_dir = [pwd, '\Input_data\'];
Output_dir = [pwd, '\Output_data\'];

% Add path of external functions 
addpath([pwd, '\third_party']);

% Initialize parameters ("Params" structure)
Params.samplingFreq = 1000;
Params.minFixDur = 0.030; 
Params.maxSaccadeAmp = 8;
Params.maxSaccadeDur = 0.070;
Params.ExcludeAboveThisVelForNoiseAnalysis = 100;
Params.PeakVelocityThreshold = 55;
Params.SaccadeSubthreshold = 45;
Params.SmallPSOVelocityThreshold = 20;

% Extract features for input files and save outputs
FileNames = dir([Input_dir, '\*.mat']);
for i = 1:numel(FileNames)
    
    % Load "ET" structure
    disp(['Loading data from: ', FileNames(i).name]);
    load([Input_dir, '\', FileNames(i).name]);
    
    % Extract features from fixations
    disp('...extracting fixation features');
    [Fixation_headers, Fixation_features] = extract_fixation_features(ET, Params);
    
    % Extract features from saccades and PSOs
    disp('...extracting saccade and PSO features');
    [Saccade_PSO_headers, Saccade_PSO_features] = extract_saccade_pso_features(ET, Params);

    % Concatenate data for storing
    headers = [Fixation_headers, Saccade_PSO_headers];
    features = [Fixation_features, Saccade_PSO_features];
    
    % Store features to .csv file
    outTable = array2table(features, 'VariableNames', headers);
    writetable(outTable, [Output_dir, FileNames(i).name(1:end-4), '_Features', '.csv']);
    
    disp('Success!');
    disp(' ');
end